var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, copyDefault, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && (copyDefault || key !== "default"))
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toCommonJS = /* @__PURE__ */ ((cache) => {
  return (module2, temp) => {
    return cache && cache.get(module2) || (temp = __reExport(__markAsModule({}), module2, 1), cache && cache.set(module2, temp), temp);
  };
})(typeof WeakMap !== "undefined" ? /* @__PURE__ */ new WeakMap() : 0);
var asn1_sequence_encoder_exports = {};
__export(asn1_sequence_encoder_exports, {
  default: () => DumbAsn1Encoder
});
var import_buffer = require("buffer");
var import_errors = require("../util/errors.js");
const tagInteger = 2;
const tagBitStr = 3;
const tagOctStr = 4;
const tagSequence = 48;
const bZero = import_buffer.Buffer.from([0]);
const bTagInteger = import_buffer.Buffer.from([tagInteger]);
const bTagBitStr = import_buffer.Buffer.from([tagBitStr]);
const bTagSequence = import_buffer.Buffer.from([tagSequence]);
const bTagOctStr = import_buffer.Buffer.from([tagOctStr]);
const encodeLength = (len) => {
  if (len < 128)
    return import_buffer.Buffer.from([len]);
  const buffer = import_buffer.Buffer.alloc(5);
  buffer.writeUInt32BE(len, 1);
  let offset = 1;
  while (buffer[offset] === 0)
    offset++;
  buffer[offset - 1] = 128 | 5 - offset;
  return buffer.slice(offset - 1);
};
const oids = /* @__PURE__ */ new Map([
  ["P-256", import_buffer.Buffer.from("06 08 2A 86 48 CE 3D 03 01 07".replace(/ /g, ""), "hex")],
  ["secp256k1", import_buffer.Buffer.from("06 05 2B 81 04 00 0A".replace(/ /g, ""), "hex")],
  ["P-384", import_buffer.Buffer.from("06 05 2B 81 04 00 22".replace(/ /g, ""), "hex")],
  ["P-521", import_buffer.Buffer.from("06 05 2B 81 04 00 23".replace(/ /g, ""), "hex")],
  ["ecPublicKey", import_buffer.Buffer.from("06 07 2A 86 48 CE 3D 02 01".replace(/ /g, ""), "hex")],
  ["X25519", import_buffer.Buffer.from("06 03 2B 65 6E".replace(/ /g, ""), "hex")],
  ["X448", import_buffer.Buffer.from("06 03 2B 65 6F".replace(/ /g, ""), "hex")],
  ["Ed25519", import_buffer.Buffer.from("06 03 2B 65 70".replace(/ /g, ""), "hex")],
  ["Ed448", import_buffer.Buffer.from("06 03 2B 65 71".replace(/ /g, ""), "hex")]
]);
class DumbAsn1Encoder {
  constructor() {
    this.length = 0;
    this.elements = [];
  }
  oidFor(oid) {
    const bOid = oids.get(oid);
    if (!bOid) {
      throw new import_errors.JOSENotSupported("Invalid or unsupported OID");
    }
    this.elements.push(bOid);
    this.length += bOid.length;
  }
  zero() {
    this.elements.push(bTagInteger, import_buffer.Buffer.from([1]), bZero);
    this.length += 3;
  }
  one() {
    this.elements.push(bTagInteger, import_buffer.Buffer.from([1]), import_buffer.Buffer.from([1]));
    this.length += 3;
  }
  unsignedInteger(integer) {
    if (integer[0] & 128) {
      const len = encodeLength(integer.length + 1);
      this.elements.push(bTagInteger, len, bZero, integer);
      this.length += 2 + len.length + integer.length;
    } else {
      let i = 0;
      while (integer[i] === 0 && (integer[i + 1] & 128) === 0)
        i++;
      const len = encodeLength(integer.length - i);
      this.elements.push(bTagInteger, encodeLength(integer.length - i), integer.slice(i));
      this.length += 1 + len.length + integer.length - i;
    }
  }
  octStr(octStr) {
    const len = encodeLength(octStr.length);
    this.elements.push(bTagOctStr, encodeLength(octStr.length), octStr);
    this.length += 1 + len.length + octStr.length;
  }
  bitStr(bitS) {
    const len = encodeLength(bitS.length + 1);
    this.elements.push(bTagBitStr, encodeLength(bitS.length + 1), bZero, bitS);
    this.length += 1 + len.length + bitS.length + 1;
  }
  add(seq) {
    this.elements.push(seq);
    this.length += seq.length;
  }
  end(tag = bTagSequence) {
    const len = encodeLength(this.length);
    return import_buffer.Buffer.concat([tag, len, ...this.elements], 1 + len.length + this.length);
  }
}
module.exports = __toCommonJS(asn1_sequence_encoder_exports);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
