var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, copyDefault, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && (copyDefault || key !== "default"))
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toESM = (module2, isNodeMode) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", !isNodeMode && module2 && module2.__esModule ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
var __toCommonJS = /* @__PURE__ */ ((cache) => {
  return (module2, temp) => {
    return cache && cache.get(module2) || (temp = __reExport(__markAsModule({}), module2, 1), cache && cache.set(module2, temp), temp);
  };
})(typeof WeakMap !== "undefined" ? /* @__PURE__ */ new WeakMap() : 0);
var generate_exports = {};
__export(generate_exports, {
  generateKeyPair: () => generateKeyPair,
  generateSecret: () => generateSecret
});
var import_crypto = require("crypto");
var import_util = require("util");
var import_random = __toESM(require("./random.js"));
var import_check_modulus_length = require("./check_modulus_length.js");
var import_errors = require("../util/errors.js");
const generate = (0, import_util.promisify)(import_crypto.generateKeyPair);
async function generateSecret(alg, options) {
  let length;
  switch (alg) {
    case "HS256":
    case "HS384":
    case "HS512":
    case "A128CBC-HS256":
    case "A192CBC-HS384":
    case "A256CBC-HS512":
      length = parseInt(alg.slice(-3), 10);
      break;
    case "A128KW":
    case "A192KW":
    case "A256KW":
    case "A128GCMKW":
    case "A192GCMKW":
    case "A256GCMKW":
    case "A128GCM":
    case "A192GCM":
    case "A256GCM":
      length = parseInt(alg.slice(1, 4), 10);
      break;
    default:
      throw new import_errors.JOSENotSupported('Invalid or unsupported JWK "alg" (Algorithm) Parameter value');
  }
  return (0, import_crypto.createSecretKey)((0, import_random.default)(new Uint8Array(length >> 3)));
}
async function generateKeyPair(alg, options) {
  var _a, _b;
  switch (alg) {
    case "RS256":
    case "RS384":
    case "RS512":
    case "PS256":
    case "PS384":
    case "PS512":
    case "RSA-OAEP":
    case "RSA-OAEP-256":
    case "RSA-OAEP-384":
    case "RSA-OAEP-512":
    case "RSA1_5": {
      const modulusLength = (_a = options == null ? void 0 : options.modulusLength) != null ? _a : 2048;
      if (typeof modulusLength !== "number" || modulusLength < 2048) {
        throw new import_errors.JOSENotSupported("Invalid or unsupported modulusLength option provided, 2048 bits or larger keys must be used");
      }
      const keypair = await generate("rsa", {
        modulusLength,
        publicExponent: 65537
      });
      (0, import_check_modulus_length.setModulusLength)(keypair.privateKey, modulusLength);
      (0, import_check_modulus_length.setModulusLength)(keypair.publicKey, modulusLength);
      return keypair;
    }
    case "ES256":
      return generate("ec", { namedCurve: "P-256" });
    case "ES256K":
      return generate("ec", { namedCurve: "secp256k1" });
    case "ES384":
      return generate("ec", { namedCurve: "P-384" });
    case "ES512":
      return generate("ec", { namedCurve: "P-521" });
    case "EdDSA": {
      switch (options == null ? void 0 : options.crv) {
        case void 0:
        case "Ed25519":
          return generate("ed25519");
        case "Ed448":
          return generate("ed448");
        default:
          throw new import_errors.JOSENotSupported("Invalid or unsupported crv option provided, supported values are Ed25519 and Ed448");
      }
    }
    case "ECDH-ES":
    case "ECDH-ES+A128KW":
    case "ECDH-ES+A192KW":
    case "ECDH-ES+A256KW":
      switch (options == null ? void 0 : options.crv) {
        case void 0:
        case "P-256":
        case "P-384":
        case "P-521":
          return generate("ec", { namedCurve: (_b = options == null ? void 0 : options.crv) != null ? _b : "P-256" });
        case "X25519":
          return generate("x25519");
        case "X448":
          return generate("x448");
        default:
          throw new import_errors.JOSENotSupported("Invalid or unsupported crv option provided, supported values are P-256, P-384, P-521, X25519, and X448");
      }
    default:
      throw new import_errors.JOSENotSupported('Invalid or unsupported JWK "alg" (Algorithm) Parameter value');
  }
}
module.exports = __toCommonJS(generate_exports);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  generateKeyPair,
  generateSecret
});
