var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __objRest = (source, exclude) => {
  var target = {};
  for (var prop in source)
    if (__hasOwnProp.call(source, prop) && exclude.indexOf(prop) < 0)
      target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(source)) {
      if (exclude.indexOf(prop) < 0 && __propIsEnum.call(source, prop))
        target[prop] = source[prop];
    }
  return target;
};
import { wrap as aesKw } from "../runtime/aeskw.js";
import * as ECDH from "../runtime/ecdhes.js";
import { encrypt as pbes2Kw } from "../runtime/pbes2kw.js";
import { encrypt as rsaEs } from "../runtime/rsaes.js";
import { encode as base64url } from "../runtime/base64url.js";
import generateCek, { bitLength as cekLength } from "../lib/cek.js";
import { JOSENotSupported } from "../util/errors.js";
import { exportJWK } from "../key/export.js";
import checkKeyType from "./check_key_type.js";
import { wrap as aesGcmKw } from "./aesgcmkw.js";
async function encryptKeyManagement(alg, enc, key, providedCek, providedParameters = {}) {
  var _a, _b;
  let encryptedKey;
  let parameters;
  let cek;
  checkKeyType(alg, key, "encrypt");
  switch (alg) {
    case "dir": {
      cek = key;
      break;
    }
    case "ECDH-ES":
    case "ECDH-ES+A128KW":
    case "ECDH-ES+A192KW":
    case "ECDH-ES+A256KW": {
      if (!ECDH.ecdhAllowed(key)) {
        throw new JOSENotSupported("ECDH-ES with the provided key is not allowed or not supported by your javascript runtime");
      }
      const { apu, apv } = providedParameters;
      let { epk: ephemeralKey } = providedParameters;
      ephemeralKey || (ephemeralKey = (await ECDH.generateEpk(key)).privateKey);
      const { x, y, crv, kty } = await exportJWK(ephemeralKey);
      const sharedSecret = await ECDH.deriveKey(key, ephemeralKey, alg === "ECDH-ES" ? enc : alg, alg === "ECDH-ES" ? cekLength(enc) : parseInt(alg.slice(-5, -2), 10), apu, apv);
      parameters = { epk: { x, crv, kty } };
      if (kty === "EC")
        parameters.epk.y = y;
      if (apu)
        parameters.apu = base64url(apu);
      if (apv)
        parameters.apv = base64url(apv);
      if (alg === "ECDH-ES") {
        cek = sharedSecret;
        break;
      }
      cek = providedCek || generateCek(enc);
      const kwAlg = alg.slice(-6);
      encryptedKey = await aesKw(kwAlg, sharedSecret, cek);
      break;
    }
    case "RSA1_5":
    case "RSA-OAEP":
    case "RSA-OAEP-256":
    case "RSA-OAEP-384":
    case "RSA-OAEP-512": {
      cek = providedCek || generateCek(enc);
      encryptedKey = await rsaEs(alg, key, cek);
      break;
    }
    case "PBES2-HS256+A128KW":
    case "PBES2-HS384+A192KW":
    case "PBES2-HS512+A256KW": {
      cek = providedCek || generateCek(enc);
      const { p2c, p2s } = providedParameters;
      _a = await pbes2Kw(alg, key, cek, p2c, p2s), { encryptedKey } = _a, parameters = __objRest(_a, ["encryptedKey"]);
      break;
    }
    case "A128KW":
    case "A192KW":
    case "A256KW": {
      cek = providedCek || generateCek(enc);
      encryptedKey = await aesKw(alg, key, cek);
      break;
    }
    case "A128GCMKW":
    case "A192GCMKW":
    case "A256GCMKW": {
      cek = providedCek || generateCek(enc);
      const { iv } = providedParameters;
      _b = await aesGcmKw(alg, key, cek, iv), { encryptedKey } = _b, parameters = __objRest(_b, ["encryptedKey"]);
      break;
    }
    default: {
      throw new JOSENotSupported('Invalid or unsupported "alg" (JWE Algorithm) header value');
    }
  }
  return { cek, encryptedKey, parameters };
}
var encrypt_key_management_default = encryptKeyManagement;
export {
  encrypt_key_management_default as default
};
