var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, copyDefault, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && (copyDefault || key !== "default"))
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toESM = (module2, isNodeMode) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", !isNodeMode && module2 && module2.__esModule ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
var __toCommonJS = /* @__PURE__ */ ((cache) => {
  return (module2, temp) => {
    return cache && cache.get(module2) || (temp = __reExport(__markAsModule({}), module2, 1), cache && cache.set(module2, temp), temp);
  };
})(typeof WeakMap !== "undefined" ? /* @__PURE__ */ new WeakMap() : 0);
var asn1_exports = {};
__export(asn1_exports, {
  fromPKCS8: () => fromPKCS8,
  fromSPKI: () => fromSPKI,
  toPKCS8: () => toPKCS8,
  toSPKI: () => toSPKI
});
var import_env = require("./env.js");
var import_webcrypto = __toESM(require("./webcrypto.js"));
var import_invalid_key_input = __toESM(require("../lib/invalid_key_input.js"));
var import_base64url = require("./base64url.js");
var import_format_pem = __toESM(require("../lib/format_pem.js"));
var import_errors = require("../util/errors.js");
var import_is_key_like = require("./is_key_like.js");
const genericExport = async (keyType, keyFormat, key) => {
  if (!(0, import_webcrypto.isCryptoKey)(key)) {
    throw new TypeError((0, import_invalid_key_input.default)(key, ...import_is_key_like.types));
  }
  if (!key.extractable) {
    throw new TypeError("CryptoKey is not extractable");
  }
  if (key.type !== keyType) {
    throw new TypeError(`key is not a ${keyType} key`);
  }
  return (0, import_format_pem.default)((0, import_base64url.encodeBase64)(new Uint8Array(await import_webcrypto.default.subtle.exportKey(keyFormat, key))), `${keyType.toUpperCase()} KEY`);
};
const toSPKI = (key) => {
  return genericExport("public", "spki", key);
};
const toPKCS8 = (key) => {
  return genericExport("private", "pkcs8", key);
};
const findOid = (keyData, oid, from = 0) => {
  if (from === 0) {
    oid.unshift(oid.length);
    oid.unshift(6);
  }
  let i = keyData.indexOf(oid[0], from);
  if (i === -1)
    return false;
  const sub = keyData.subarray(i, i + oid.length);
  if (sub.length !== oid.length)
    return false;
  return sub.every((value, index) => value === oid[index]) || findOid(keyData, oid, i + 1);
};
const getNamedCurve = (keyData) => {
  switch (true) {
    case findOid(keyData, [42, 134, 72, 206, 61, 3, 1, 7]):
      return "P-256";
    case findOid(keyData, [43, 129, 4, 0, 34]):
      return "P-384";
    case findOid(keyData, [43, 129, 4, 0, 35]):
      return "P-521";
    case (((0, import_env.isCloudflareWorkers)() || (0, import_env.isNodeJs)()) && findOid(keyData, [43, 101, 112])):
      return "Ed25519";
    case ((0, import_env.isNodeJs)() && findOid(keyData, [43, 101, 113])):
      return "Ed448";
    default:
      throw new import_errors.JOSENotSupported("Invalid or unsupported EC Key Curve or OKP Key Sub Type");
  }
};
const genericImport = async (replace, keyFormat, pem, alg, options) => {
  let algorithm;
  let keyUsages;
  const keyData = new Uint8Array(atob(pem.replace(replace, "")).split("").map((c) => c.charCodeAt(0)));
  const isPublic = keyFormat === "spki";
  switch (alg) {
    case "PS256":
    case "PS384":
    case "PS512":
      algorithm = { name: "RSA-PSS", hash: `SHA-${alg.slice(-3)}` };
      keyUsages = isPublic ? ["verify"] : ["sign"];
      break;
    case "RS256":
    case "RS384":
    case "RS512":
      algorithm = { name: "RSASSA-PKCS1-v1_5", hash: `SHA-${alg.slice(-3)}` };
      keyUsages = isPublic ? ["verify"] : ["sign"];
      break;
    case "RSA-OAEP":
    case "RSA-OAEP-256":
    case "RSA-OAEP-384":
    case "RSA-OAEP-512":
      algorithm = {
        name: "RSA-OAEP",
        hash: `SHA-${parseInt(alg.slice(-3), 10) || 1}`
      };
      keyUsages = isPublic ? ["encrypt", "wrapKey"] : ["decrypt", "unwrapKey"];
      break;
    case "ES256":
      algorithm = { name: "ECDSA", namedCurve: "P-256" };
      keyUsages = isPublic ? ["verify"] : ["sign"];
      break;
    case "ES384":
      algorithm = { name: "ECDSA", namedCurve: "P-384" };
      keyUsages = isPublic ? ["verify"] : ["sign"];
      break;
    case "ES512":
      algorithm = { name: "ECDSA", namedCurve: "P-521" };
      keyUsages = isPublic ? ["verify"] : ["sign"];
      break;
    case "ECDH-ES":
    case "ECDH-ES+A128KW":
    case "ECDH-ES+A192KW":
    case "ECDH-ES+A256KW":
      algorithm = { name: "ECDH", namedCurve: getNamedCurve(keyData) };
      keyUsages = isPublic ? [] : ["deriveBits"];
      break;
    case (((0, import_env.isCloudflareWorkers)() || (0, import_env.isNodeJs)()) && "EdDSA"):
      const namedCurve = getNamedCurve(keyData).toUpperCase();
      algorithm = { name: `NODE-${namedCurve}`, namedCurve: `NODE-${namedCurve}` };
      keyUsages = isPublic ? ["verify"] : ["sign"];
      break;
    default:
      throw new import_errors.JOSENotSupported('Invalid or unsupported "alg" (Algorithm) value');
  }
  return import_webcrypto.default.subtle.importKey(keyFormat, keyData, algorithm, options?.extractable ?? false, keyUsages);
};
const fromPKCS8 = (pem, alg, options) => {
  return genericImport(/(?:-----(?:BEGIN|END) PRIVATE KEY-----|\s)/g, "pkcs8", pem, alg, options);
};
const fromSPKI = (pem, alg, options) => {
  return genericImport(/(?:-----(?:BEGIN|END) PUBLIC KEY-----|\s)/g, "spki", pem, alg, options);
};
module.exports = __toCommonJS(asn1_exports);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  fromPKCS8,
  fromSPKI,
  toPKCS8,
  toSPKI
});
