var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, copyDefault, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && (copyDefault || key !== "default"))
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toESM = (module2, isNodeMode) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", !isNodeMode && module2 && module2.__esModule ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
var __toCommonJS = /* @__PURE__ */ ((cache) => {
  return (module2, temp) => {
    return cache && cache.get(module2) || (temp = __reExport(__markAsModule({}), module2, 1), cache && cache.set(module2, temp), temp);
  };
})(typeof WeakMap !== "undefined" ? /* @__PURE__ */ new WeakMap() : 0);
var decrypt_exports = {};
__export(decrypt_exports, {
  default: () => decrypt_default
});
var import_buffer_utils = require("../lib/buffer_utils.js");
var import_check_iv_length = __toESM(require("../lib/check_iv_length.js"));
var import_check_cek_length = __toESM(require("./check_cek_length.js"));
var import_timing_safe_equal = __toESM(require("./timing_safe_equal.js"));
var import_errors = require("../util/errors.js");
var import_webcrypto = __toESM(require("./webcrypto.js"));
var import_crypto_key = require("../lib/crypto_key.js");
var import_invalid_key_input = __toESM(require("../lib/invalid_key_input.js"));
var import_is_key_like = require("./is_key_like.js");
async function cbcDecrypt(enc, cek, ciphertext, iv, tag, aad) {
  if (!(cek instanceof Uint8Array)) {
    throw new TypeError((0, import_invalid_key_input.default)(cek, "Uint8Array"));
  }
  const keySize = parseInt(enc.slice(1, 4), 10);
  const encKey = await import_webcrypto.default.subtle.importKey("raw", cek.subarray(keySize >> 3), "AES-CBC", false, ["decrypt"]);
  const macKey = await import_webcrypto.default.subtle.importKey("raw", cek.subarray(0, keySize >> 3), {
    hash: `SHA-${keySize << 1}`,
    name: "HMAC"
  }, false, ["sign"]);
  const macData = (0, import_buffer_utils.concat)(aad, iv, ciphertext, (0, import_buffer_utils.uint64be)(aad.length << 3));
  const expectedTag = new Uint8Array((await import_webcrypto.default.subtle.sign("HMAC", macKey, macData)).slice(0, keySize >> 3));
  let macCheckPassed;
  try {
    macCheckPassed = (0, import_timing_safe_equal.default)(tag, expectedTag);
  } catch {
  }
  if (!macCheckPassed) {
    throw new import_errors.JWEDecryptionFailed();
  }
  let plaintext;
  try {
    plaintext = new Uint8Array(await import_webcrypto.default.subtle.decrypt({ iv, name: "AES-CBC" }, encKey, ciphertext));
  } catch {
  }
  if (!plaintext) {
    throw new import_errors.JWEDecryptionFailed();
  }
  return plaintext;
}
async function gcmDecrypt(enc, cek, ciphertext, iv, tag, aad) {
  let encKey;
  if (cek instanceof Uint8Array) {
    encKey = await import_webcrypto.default.subtle.importKey("raw", cek, "AES-GCM", false, ["decrypt"]);
  } else {
    (0, import_crypto_key.checkEncCryptoKey)(cek, enc, "decrypt");
    encKey = cek;
  }
  try {
    return new Uint8Array(await import_webcrypto.default.subtle.decrypt({
      additionalData: aad,
      iv,
      name: "AES-GCM",
      tagLength: 128
    }, encKey, (0, import_buffer_utils.concat)(ciphertext, tag)));
  } catch {
    throw new import_errors.JWEDecryptionFailed();
  }
}
const decrypt = async (enc, cek, ciphertext, iv, tag, aad) => {
  if (!(0, import_webcrypto.isCryptoKey)(cek) && !(cek instanceof Uint8Array)) {
    throw new TypeError((0, import_invalid_key_input.default)(cek, ...import_is_key_like.types, "Uint8Array"));
  }
  (0, import_check_iv_length.default)(enc, iv);
  switch (enc) {
    case "A128CBC-HS256":
    case "A192CBC-HS384":
    case "A256CBC-HS512":
      if (cek instanceof Uint8Array)
        (0, import_check_cek_length.default)(cek, parseInt(enc.slice(-3), 10));
      return cbcDecrypt(enc, cek, ciphertext, iv, tag, aad);
    case "A128GCM":
    case "A192GCM":
    case "A256GCM":
      if (cek instanceof Uint8Array)
        (0, import_check_cek_length.default)(cek, parseInt(enc.slice(1, 4), 10));
      return gcmDecrypt(enc, cek, ciphertext, iv, tag, aad);
    default:
      throw new import_errors.JOSENotSupported("Unsupported JWE Content Encryption Algorithm");
  }
};
var decrypt_default = decrypt;
module.exports = __toCommonJS(decrypt_exports);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
